{$APPTYPE CONSOLE}

program DemoVideoCallBacks;

uses
  Windows,
  Graphics,
  PasLibVlcUnit in '..\..\source\PasLibVlcUnit.pas';

type
  PCtx = ^TCtx;
  TCtx = record
    buff : Pointer;
    bmpi : Graphics.TBitmap;
    bmpc : Cardinal;
    video_w : LongWord;
    video_h : LongWord;
  end;

const
  DEMO_FILE = '..\..\_testFiles\Maximize.mp4';
  MAX_ARGS  = 256;

var
  argv : packed array[0..MAX_ARGS-1] of AnsiString;
  args : packed array[0..MAX_ARGS-1] of PAnsiChar;
  argc : Integer;

  p_li : libvlc_instance_t_ptr;
  p_md : libvlc_media_t_ptr;
  p_mi : libvlc_media_player_t_ptr;

  ctx : TCtx;

procedure AddArg(value : AnsiString);
begin
  if (argc < MAX_ARGS) then
  begin
    argv[argc] := value;
    args[argc] := PAnsiChar(argv[argc]);
    Inc(argc);
  end;
end;  

function libvlc_video_lock_cb(ptr : Pointer; var planes : Pointer) : Pointer; cdecl;
var
  ctx : PCtx;
begin
  ctx := PCtx(ptr);
  planes := Pointer(((Cardinal(ctx^.buff) + 31) shr 5) shl 5);
  Result := ctx^.buff;
end;

procedure libvlc_video_unlock_cb(ptr : Pointer; picture : Pointer; planes : Pointer); cdecl;
begin
end;

procedure libvlc_video_display_cb_RV32(ptr : Pointer; picture : Pointer); cdecl;
var
  y        : Integer;
  src      : PByte;
  fileNumb : ShortString;
  fileName : ShortString;
  ctx      : PCtx;
  len      : LongWord;
begin
  ctx := PCtx(ptr);

  src := Pointer(((Cardinal(ctx^.buff) + 31) shr 5) shl 5);

  len := ctx^.video_w * 4;
  for y := 0 to ctx^.video_h - 1 do
  begin
    Move(src^, ctx^.bmpi.ScanLine[y]^, len);
    Inc(src, len);
  end;
  Inc(ctx^.bmpc);
  Str(ctx^.bmpc, fileNumb);

  fileName := Copy('000000', 1, 6 - Length(fileNumb)) + fileNumb + '.bmp';
  WriteLn(fileName);
  
  ctx^.bmpi.SaveToFile(fileName);
end;

function libvlc_video_format_cb(var ptr : Pointer; chroma : PAnsiChar; var width : LongWord; var height : LongWord; var pitches : TVCBPitches; var lines : TVCBLines) : LongWord; cdecl;
var
  ctx : PCtx;
  idx : Integer;
begin
  ctx := PCtx(ptr);

  with ctx^ do
  begin
    video_w := width;
    video_h := height;
    ReAllocMem(buff, (((width + 31) shr 5) shl 5) * (((height + 31) shr 5) shl 5) * 4 + 32);
    bmpi.SetSize(width, height);
  end;

  chroma^ := AnsiChar('R'); Inc(chroma);
  chroma^ := AnsiChar('V'); Inc(chroma);
  chroma^ := AnsiChar('3'); Inc(chroma);
  chroma^ := AnsiChar('2'); Inc(chroma);

  (*
   * Furthermore, we recommend that pitches and lines be multiple of 32
   * to not break assumption that might be made by various optimizations
   * in the video decoders, video filters and/or video converters.
   *)
  pitches[0] := (((width * 4 + 31) shr 5) shl 5); // width * 4;
  lines[0] := (((height + 31) shr 5) shl 5);

  for idx := 1 to VOUT_MAX_PLANES - 1 do
  begin
    pitches[idx] := pitches[0];
    lines[idx] := lines[0];
  end;

  Result := 1;
end;

procedure libvlc_video_cleanup_cb(ptr : Pointer); cdecl;
var
  ctx : PCtx;
begin
  ctx := PCtx(ptr);
  with ctx^ do
  begin
    if (buff <> NIL) then
    begin
      FreeMem(buff);
    end;
    bmpi.Free;
  end;
end;

begin
  FillChar(ctx, sizeof(ctx), 0);

  with ctx do
  begin
    bmpc := 0;

    bmpi := Graphics.TBitmap.Create;
    bmpi.PixelFormat := pf32bit;
    bmpi.SetSize(video_w, video_h);
  end;

  libvlc_dynamic_dll_init();

  argc := 0;
  AddArg(libvlc_dynamic_dll_path);
  AddArg('--intf=dummy');
  AddArg('--ignore-config');
  AddArg('--quiet');
  AddArg('--no-video-title-show');
  AddArg('--vout=vdummy');
  AddArg('--aout=adummy');

  p_li := libvlc_new(argc, @args);
  p_md := libvlc_media_new_path(p_li, PAnsiChar(UTF8Encode(DEMO_FILE)));
  libvlc_media_parse(p_md);
  
  p_mi := libvlc_media_player_new_from_media(p_md);

  libvlc_video_set_callbacks(p_mi, libvlc_video_lock_cb, libvlc_video_unlock_cb, libvlc_video_display_cb_RV32, @ctx);

  // use this for set video size to your needs
//  libvlc_video_set_format(p_mi, 'RV32', ctx.video_w, ctx.video_h, ctx.video_w * 4);

  // use this for handle change video size etc.
  libvlc_video_set_format_callbacks(p_mi, libvlc_video_format_cb, libvlc_video_cleanup_cb);

  libvlc_media_player_play(p_mi);
  
  while (libvlc_media_player_get_state(p_mi) <> libvlc_Ended) do
  begin
    Sleep(50);
  end;

  libvlc_media_release(p_md);
  libvlc_media_player_release(p_mi);
  libvlc_release(p_li);
end.
